/*******************************************************************************
 * Copyright 2013 Francesco Cina'
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package com.jporm.query.save;

import com.jporm.mapper.ServiceCatalog;
import com.jporm.query.crud.executor.SaveOrUpdateType;

/**
 * 
 * @author Francesco Cina
 *
 * 10/lug/2011
 */
public class SaveQueryOrm<BEAN> implements SaveQuery<BEAN> {

    private int _queryTimeout = 0;
    private final Class<BEAN> clazz;
    private final BEAN bean;
    private boolean _cascade = false;
    private SaveOrUpdateType _saveOrUpdateType = SaveOrUpdateType.SAVE;
    private final ServiceCatalog serviceCatalog;

    public SaveQueryOrm(final BEAN bean, final ServiceCatalog serviceCatalog) {
        this.bean = bean;
        this.serviceCatalog = serviceCatalog;
        this.clazz = (Class<BEAN>) bean.getClass();
    }

    @Override
    public BEAN now() {
        return serviceCatalog.getOrmQueryExecutor().saveOrUpdate().save(bean, clazz, _cascade, _saveOrUpdateType, _queryTimeout);
    }

    @Override
    public SaveQuery<BEAN> queryTimeout(final int queryTimeout) {
        this._queryTimeout = queryTimeout;
        return this;
    }

    @Override
    public int getQueryTimeout() {
        return this._queryTimeout;
    }

    /**
     * @param cascade
     * @return
     */
    @Override
    public SaveQuery<BEAN> cascade(final boolean cascade) {
        this._cascade = cascade;
        return this;
    }

    @Override
    public SaveQuery<BEAN> saveOrUpdate(final SaveOrUpdateType saveOrUpdateType) {
        _saveOrUpdateType = saveOrUpdateType;
        return this;
    }

}
